require "Alimento/alimento"

#Clase Alimento que contiene el nombre, proteinas, glucidos y lipidos de un alimento
class Alimentos

    include Comparable

    #Inicializa los valores
    def initialize(nom, pro, glu, lip)
        @nombre, @proteinas, @glucidos, @lipidos = nom, pro, glu, lip
    end

    #Obtienes nombre del alimento
    def nombre_to_s
        @nombre
    end

    #Obtienes proteinas del alimento
    def proteinas_to_i
        @proteinas
    end

    #Obtienes glucidos del alimento
    def glucidos_to_i
        @glucidos
    end

    #Obtienes lipidos del alimento
    def lipidos_to_i
        @lipidos
    end

    #Obtienes toda la informacion del alimento
    def informacion_to_s
        "%-24s%-16s%-16s%-16s" % [@nombre,@glucidos,@proteinas,@lipidos]
    end

    #Obtienes valor energetico del alimento
    def valor_energetico
        ((@proteinas*4)+(@glucidos*4)+(@lipidos*9))
    end

    #Funcion para el modulo comparable
    def <=>(other)
        self.valor_energetico <=> other.valor_energetico
    end
end

#Clase Grupo_alimentos que hereda de alimentos
class Grupo_alimentos < Alimentos

    #Inicializa los valores
    def initialize(nom, pro, glu, lip, grupo)
        super(nom, pro, glu, lip)
        @nombre_grupo = grupo
    end

    #Obtienes nombre del grupo del alimento
    def to_s
        @nombre_grupo
    end
end

class Alimento_IG < Alimentos

    include Enumerable
    #Inicializa los valores
    def initialize(nom, pro, glu, lip, valores)
        super(nom, pro, glu, lip)
        @ign = valores
    end

    def each
      yield @ign[0]
      yield @ign[1]
    end
end

class Plato

  @@medidas = { "cucharadas"=>2,"piezas pequeñas"=>1,"taza"=>3,"cucharón"=>3,"pieza"=>4,"cucharada"=>2 }

  def initialize(name, list, &block)
    @nombre = name
    @lista_alimentos = list
    @vegetal = []
    @fruta = []
    @cereal = []
    @proteina = []
    @aceite = []
    @valor_energetico = 0

    if block_given?
      if block.arity == 1
        yield self
      else
       instance_eval(&block)
      end
    end
  end

  def to_s
      puts"#{@nombre}"
      puts"================================================================="
      puts"Composicion nutricional:"
      puts"\t\t\tglucidos\tproteınas\tlıpidos\t\tvalor energetico"
      @vegetal.map { |e| printf"#{e[0][:value].informacion_to_s}#{e[1]}\n"}
      @fruta.map { |e| printf"#{e[0][:value].informacion_to_s}#{e[1]}\n"}
      @cereal.map { |e| printf"#{e[0][:value].informacion_to_s}#{e[1]}\n"}
      @proteina.map { |e| printf"#{e[0][:value].informacion_to_s}#{e[1]}\n"}
      @aceite.map { |e| printf"#{e[0][:value].informacion_to_s}#{e[1]}\n"}
      puts"Valor energetico total %66s" % @valor_energetico
  end

  def vegetal(name, options = {})
    cantidad = 0
    alimento = @lista_alimentos.find { |x| name == x[:value].nombre_to_s }

    if options[:gramos]
       cantidad = options[:gramos]
    else
        porcion = options[:porcion].split(" ", 2)
        cantidad = @@medidas[porcion[1]]
        cantidad = cantidad * porcion[0].to_r.to_f
    end
    @valor_energetico += cantidad * alimento[:value].valor_energetico()
    @vegetal.push([alimento, (cantidad * alimento[:value].valor_energetico).round(2)])
  end

  def fruta(name, options = {})
    cantidad = 0
    alimento = @lista_alimentos.find { |x| name == x[:value].nombre_to_s }

    if options[:gramos]
       cantidad = options[:gramos]
    else
        porcion = options[:porcion].split(" ", 2)
        cantidad = @@medidas[porcion[1]]
        cantidad = cantidad * porcion[0].to_r.to_f
    end
    @valor_energetico += cantidad * alimento[:value].valor_energetico()
    @fruta.push([alimento, (cantidad * alimento[:value].valor_energetico).round(2)])
  end

  def cereal(name, options = {})
    cantidad = 0
    alimento = @lista_alimentos.find { |x| name == x[:value].nombre_to_s }

    if options[:gramos]
       cantidad = options[:gramos]
    else
        porcion = options[:porcion].split(" ", 2)
        cantidad = @@medidas[porcion[1]]
        cantidad = cantidad * porcion[0].to_r.to_f
    end
    @valor_energetico += cantidad * alimento[:value].valor_energetico()
    @cereal.push([alimento, (cantidad * alimento[:value].valor_energetico).round(2)])
  end

  def proteina(name, options = {})
    cantidad = 0
    alimento = @lista_alimentos.find { |x| name == x[:value].nombre_to_s }

    if options[:gramos]
       cantidad = options[:gramos]
    else
        porcion = options[:porcion].split(" ", 2)
        cantidad = @@medidas[porcion[1]]
        cantidad = cantidad * porcion[0].to_r.to_f
    end
    @valor_energetico += cantidad * alimento[:value].valor_energetico()
    @proteina.push([alimento, (cantidad * alimento[:value].valor_energetico).round(2)])
  end

  def aceite(name, options = {})
    cantidad = 0
    alimento = @lista_alimentos.find { |x| name == x[:value].nombre_to_s }

    if options[:gramos]
       cantidad = options[:gramos]
    else
        porcion = options[:porcion].split(" ", 2)
        cantidad = @@medidas[porcion[1]]
        cantidad = cantidad * porcion[0].to_r.to_f
    end
    @valor_energetico += cantidad * alimento[:value].valor_energetico()
    @aceite.push([alimento, (cantidad * alimento[:value].valor_energetico).round(2)])
  end
end
